<?php
/*
 +--------------------------------------------------------------------+
 | CiviCRM version 4.2                                                |
 +--------------------------------------------------------------------+
 | Copyright CiviCRM LLC (c) 2004-2012                                |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the GNU Affero General Public License           |
 | Version 3, 19 November 2007 and the CiviCRM Licensing Exception.   |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the GNU Affero General Public License for more details.        |
 |                                                                    |
 | You should have received a copy of the GNU Affero General Public   |
 | License and the CiviCRM Licensing Exception along                  |
 | with this program; if not, contact CiviCRM LLC                     |
 | at info[AT]civicrm[DOT]org. If you have questions about the        |
 | GNU Affero General Public License or the licensing of CiviCRM,     |
 | see the CiviCRM license FAQ at http://civicrm.org/licensing        |
 +--------------------------------------------------------------------+
*/

/**
 *
 * @package CRM
 * @copyright CiviCRM LLC (c) 2004-2012
 * $Id$
 *
 */

/**
 * This class generates form components for custom data
 *
 * It delegates the work to lower level subclasses and integrates the changes
 * back in. It also uses a lot of functionality with the CRM API's, so any change
 * made here could potentially affect the API etc. Be careful, be aware, use unit tests.
 *
 */
class CRM_Profile_Form extends CRM_Core_Form {
  CONST MODE_REGISTER = 1, MODE_SEARCH = 2, MODE_CREATE = 4, MODE_EDIT = 8;

  protected $_mode;

  protected $_skipPermission = FALSE;

  /**
   * The contact id that we are editing
   *
   * @var int
   */
  protected $_id;

  /**
   * The group id that we are editing
   *
   * @var int
   */
  protected $_gid;

  /**
   * The group id that we are passing in url
   *
   * @var int
   */
  public $_grid;

  /**
   * The title of the category we are editing
   *
   * @var string
   */
  protected $_title;

  /**
   * the fields needed to build this form
   *
   * @var array
   */
  public $_fields;

  /**
   * to store contact details
   *
   * @var array
   */
  protected $_contact;

  /**
   * to store group_id of the group which is to be assigned to the contact
   *
   * @var int
   */
  protected $_addToGroupID;

  /**
   * Do we allow updates of the contact
   *
   * @var int
   */
  public $_isUpdateDupe = 0;

  public $_isAddCaptcha = FALSE;

  protected $_isPermissionedChecksum = FALSE;

  /**
   * THe context from which we came from, allows us to go there if redirect not set
   *
   * @var string
   */
  protected $_context;

  /**
   * THe contact type for registration case
   *
   * @var string
   */
  protected $_ctype = NULL;

  protected $_defaults = NULL;

  /**
   * Store profile ids if multiple profile ids are passed using comma separated.
   * Currently lets implement this functionality only for dialog mode
   */
  protected $_profileIds = array();

  /**
   * Contact profile having activity fields?
   *
   * @var string
   */
  protected $_isContactActivityProfile = FALSE;

  /**
   * Activity Id connected to the profile
   *
   * @var string
   */
  protected $_activityId = NULL;

  /**
   * pre processing work done here.
   *
   * gets session variables for table name, id of entity in table, type of entity and stores them.
   *
   * @param
   *
   * @return void
   *
   * @access public
   */ function preProcess() {

    $this->_id         = $this->get('id');
    $this->_gid        = $this->get('gid');
    $this->_profileIds = $this->get('profileIds');
    $this->_grid       = CRM_Utils_Request::retrieve('grid', 'Integer', $this);
    $this->_context    = CRM_Utils_Request::retrieve('context', 'String', $this);

    $this->_duplicateButtonName = $this->getButtonName('upload', 'duplicate');

    $gids = explode(',', CRM_Utils_Request::retrieve('gid', 'String', CRM_Core_DAO::$_nullObject, FALSE, 0, 'GET'));

    if ((count($gids) > 1) && !$this->_profileIds && empty($this->_profileIds)) {
      if (!empty($gids)) {
        foreach ($gids as $pfId) {
          $this->_profileIds[] = CRM_Utils_Type::escape($pfId, 'Positive');
        }
      }

      // check if we are rendering mixed profiles
      if (CRM_Core_BAO_UFGroup::checkForMixProfiles($this->_profileIds)) {
        CRM_Core_Error::fatal(ts('You cannot combine profiles of multiple types.'));
      }

      // for now consider 1'st profile as primary profile and validate it
      // i.e check for profile type etc.
      // FIX ME: validations for other than primary
      $this->_gid = $this->_profileIds[0];
      $this->set('gid', $this->_gid);
      $this->set('profileIds', $this->_profileIds);
    }

    if (!$this->_gid) {
      $this->_gid = CRM_Utils_Request::retrieve('gid', 'Positive', $this, FALSE, 0, 'GET');
    }

    $this->_activityId = CRM_Utils_Request::retrieve('aid', 'Positive', $this, FALSE, 0, 'GET');
    if (is_numeric($this->_activityId)) {
      $latestRevisionId = CRM_Activity_BAO_Activity::getLatestActivityId($this->_activityId);
      if ($latestRevisionId) {
        $this->_activityId = $latestRevisionId;
      }
    }
    $this->_isContactActivityProfile = CRM_Core_BAO_UFField::checkContactActivityProfileType($this->_gid);

    //get values for captch and dupe update.
    if ($this->_gid) {
      $dao = new CRM_Core_DAO_UFGroup();
      $dao->id = $this->_gid;
      if ($dao->find(TRUE)) {
        $this->_isUpdateDupe = $dao->is_update_dupe;
        $this->_isAddCaptcha = $dao->add_captcha;
      }
      $dao->free();
    }

    if (empty($this->_profileIds)) {
      $gids = $this->_gid;
    }
    else {
      $gids = $this->_profileIds;
    }

    // if we dont have a gid use the default, else just use that specific gid
    if (($this->_mode == self::MODE_REGISTER || $this->_mode == self::MODE_CREATE) && !$this->_gid) {
      $this->_ctype = CRM_Utils_Request::retrieve('ctype', 'String', $this, FALSE, 'Individual', 'REQUEST');
      $this->_fields = CRM_Core_BAO_UFGroup::getRegistrationFields($this->_action, $this->_mode, $this->_ctype);
    }
    elseif ($this->_mode == self::MODE_SEARCH) {
      $this->_fields = CRM_Core_BAO_UFGroup::getListingFields($this->_action,
        CRM_Core_BAO_UFGroup::PUBLIC_VISIBILITY | CRM_Core_BAO_UFGroup::LISTINGS_VISIBILITY,
        FALSE,
        $gids,
        TRUE, NULL,
        $this->_skipPermission,
        CRM_Core_Permission::SEARCH
      );
    }
    else {
      $this->_fields = CRM_Core_BAO_UFGroup::getFields($gids, FALSE, NULL,
        NULL, NULL,
        FALSE, NULL,
        $this->_skipPermission,
        NULL,
        ($this->_action == CRM_Core_Action::ADD) ? CRM_Core_Permission::CREATE : CRM_Core_Permission::EDIT
      );

      // is profile double-opt in?
      if (CRM_Utils_Array::value('group', $this->_fields) &&
        CRM_Core_BAO_UFGroup::isProfileDoubleOptin()
      ) {
        $emailField = FALSE;
        foreach ($this->_fields as $name => $values) {
          if (substr($name, 0, 6) == 'email-') {
            $emailField = TRUE;
          }
        }

        if (!$emailField) {
          $session = CRM_Core_Session::singleton();
          $status = ts("Email field should be included in profile if you want to use Group(s) when Profile double-opt in process is enabled.");
          $session->setStatus($status);
        }
      }
    }

    if (!is_array($this->_fields)) {
      $session = CRM_Core_Session::singleton();
      CRM_Core_Session::setStatus(ts('This feature is not currently available.'));
      return CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm', 'reset=1'));
    }

    if ($this->_mode != self::MODE_SEARCH) {
      CRM_Core_BAO_UFGroup::setRegisterDefaults($this->_fields, $defaults);
      $this->setDefaults($defaults);
    }
  }

  /**
   * This function sets the default values for the form. Note that in edit/view mode
   * the default values are retrieved from the database
   *
   * @access public
   *
   * @return void
   */
  function setDefaultsValues() {
    $this->_defaults = array();
    if ($this->_id) {
      if ($this->_isContactActivityProfile) {
        $contactFields = $activityFields = array();
        foreach ($this->_fields as $fieldName => $field) {
          if (CRM_Utils_Array::value('field_type', $field) == 'Activity') {
            $activityFields[$fieldName] = $field;
          }
          else {
            $contactFields[$fieldName] = $field;
          }
        }

        CRM_Core_BAO_UFGroup::setProfileDefaults($this->_id, $contactFields, $this->_defaults, TRUE);
        if ($this->_activityId) {
          CRM_Core_BAO_UFGroup::setComponentDefaults($activityFields, $this->_activityId, 'Activity', $this->_defaults, TRUE);
        }
      }
      else {
        CRM_Core_BAO_UFGroup::setProfileDefaults($this->_id, $this->_fields, $this->_defaults, TRUE);
      }
    }

    //set custom field defaults
    foreach ($this->_fields as $name => $field) {
      if ($customFieldID = CRM_Core_BAO_CustomField::getKeyID($name)) {
        $htmlType = $field['html_type'];

        if ((!isset($this->_defaults[$name]) || $htmlType == 'File') &&
          (CRM_Utils_Array::value('field_type', $field) != 'Activity')
        ) {
          CRM_Core_BAO_CustomField::setProfileDefaults($customFieldID,
            $name,
            $this->_defaults,
            $this->_id,
            $this->_mode
          );
        }

        if ($htmlType == 'File') {
          $entityId = $this->_id;
          if (CRM_Utils_Array::value('field_type', $field) == 'Activity' &&
            $this->_activityId
          ) {
            $entityId = $this->_activityId;
          }
          $url = CRM_Core_BAO_CustomField::getFileURL($entityId, $customFieldID);

          if ($url) {
            $customFiles[$field['name']]['displayURL'] = ts("Attached File") . ": {$url['file_url']}";

            $deleteExtra = ts("Are you sure you want to delete attached file?");
            $fileId      = $url['file_id'];
            $deleteURL   = CRM_Utils_System::url('civicrm/file',
              "reset=1&id={$fileId}&eid=$entityId&fid={$customFieldID}&action=delete"
            );
            $text = ts("Delete Attached File");
            $customFiles[$field['name']]['deleteURL'] = "<a href=\"{$deleteURL}\" onclick = \"if (confirm( ' $deleteExtra ' )) this.href+='&amp;confirmed=1'; else return false;\">$text</a>";

            // also delete the required rule that we've set on the form element
            $this->removeFileRequiredRules($field['name']);
          }
        }
      }
    }
    if (isset($customFiles)) {
      $this->assign('customFiles', $customFiles);
    }

    if (CRM_Utils_Array::value('image_URL', $this->_defaults)) {
      list($imageWidth, $imageHeight) = getimagesize($this->_defaults['image_URL']);
      list($imageThumbWidth, $imageThumbHeight) = CRM_Contact_BAO_Contact::getThumbSize($imageWidth, $imageHeight);
      $this->assign("imageWidth", $imageWidth);
      $this->assign("imageHeight", $imageHeight);
      $this->assign("imageThumbWidth", $imageThumbWidth);
      $this->assign("imageThumbHeight", $imageThumbHeight);
      $this->assign("imageURL", $this->_defaults['image_URL']);
    }

    if (array_key_exists('contact_sub_type', $this->_defaults) &&
      !empty($this->_defaults['contact_sub_type'])
    ) {
      $this->_defaults['contact_sub_type'] = explode(CRM_Core_DAO::VALUE_SEPARATOR,
        trim($this->_defaults['contact_sub_type'], CRM_Core_DAO::VALUE_SEPARATOR)
      );
    }

    $this->setDefaults($this->_defaults);
  }

  /**
   * Function to actually build the form
   *
   * @return void
   * @access public
   */
  public function buildQuickForm() {
    //lets have single status message, CRM-4363
    $return = FALSE;
    $statusMessage = NULL;

    CRM_Core_BAO_Address::checkContactSharedAddressFields($this->_fields, $this->_id);
    //we should not allow component and mix profiles in search mode
    if ($this->_mode != self::MODE_REGISTER) {
      //check for mix profile fields (eg:  individual + other contact type)
      if (CRM_Core_BAO_UFField::checkProfileType($this->_gid)) {

        if (($this->_mode & self::MODE_EDIT) && $this->_isContactActivityProfile) {
          $errors = self::validateContactActivityProfile($this->_activityId, $this->_id, $this->_gid);
          if (!empty($errors)) {
            $statusMessage = array_pop($errors);
            $return = TRUE;
          }
        }
        else {
          $statusMessage = ts('Profile search, view and edit are not supported for Profiles which include fields for more than one record type.');
          $return = TRUE;
        }
      }

      $profileType = CRM_Core_BAO_UFField::getProfileType($this->_gid);

      if ($this->_id) {
        $contactTypes = CRM_Contact_BAO_Contact::getContactTypes($this->_id);
        $contactType  = $contactTypes[0];

        array_shift($contactTypes);
        $contactSubtypes = $contactTypes;

        $profileSubType = FALSE;
        if (CRM_Contact_BAO_ContactType::isaSubType($profileType)) {
          $profileSubType = $profileType;
          $profileType = CRM_Contact_BAO_ContactType::getBasicType($profileType);
        }

        if (($profileType != 'Contact' && !$this->_isContactActivityProfile) &&
            (($profileSubType && !empty($contactSubtypes) && (!in_array($profileSubType,$contactSubtypes))) ||
            ($profileType != $contactType)
          )
        ) {
          $return = TRUE;
          if (!$statusMessage) {
            $statusMessage = ts("This profile is configured for contact type '%1'. It cannot be used to edit contacts of other types.", array(1 => $profileSubType ? $profileSubType : $profileType));
          }
        }
      }

      if (in_array($profileType, array(
        "Membership", "Participant", "Contribution"))) {
        $return = TRUE;
        if (!$statusMessage) {
          $statusMessage = ts('Profile is not configured for the selected action.');
        }
      }
    }

    //lets have sigle status message,
    $this->assign('statusMessage', $statusMessage);
    if ($return) {
      return FALSE;
    }

    $sBlocks = array();
    $hBlocks = array();
    $config  = CRM_Core_Config::singleton();

    $this->assign('id', $this->_id);
    $this->assign('mode', $this->_mode);
    $this->assign('action', $this->_action);
    $this->assign_by_ref('fields', $this->_fields);
    $this->assign('fieldset', (isset($this->_fieldset)) ? $this->_fieldset : "");

    // do we need inactive options ?
    if ($this->_action & CRM_Core_Action::VIEW) {
      $inactiveNeeded = TRUE;
    }
    else {
      $inactiveNeeded = FALSE;
    }

    $session = CRM_Core_Session::singleton();

    // should we restrict what we display
    $admin = TRUE;
    if ($this->_mode == self::MODE_EDIT) {
      $admin = FALSE;
      // show all fields that are visibile:
      // if we are a admin OR the same user OR acl-user with access to the profile
      // or we have checksum access to this contact (i.e. the user without a login) - CRM-5909
      if (CRM_Core_Permission::check('administer users') ||
        $this->_id == $session->get('userID') ||
        $this->_isPermissionedChecksum ||
        in_array($this->_gid,
          CRM_ACL_API::group(CRM_Core_Permission::EDIT,
            NULL,
            'civicrm_uf_group',
            CRM_Core_PseudoConstant::ufGroup()
          )
        )
      ) {
        $admin = TRUE;
      }
    }

    $userID = $session->get('userID');
    // if false, user is not logged-in.
    $anonUser = FALSE;
    if (!$userID) {
      $defaultLocationType = CRM_Core_BAO_LocationType::getDefault();
      $primaryLocationType = $defaultLocationType->id;
      $anonUser            = TRUE;
      $this->assign('anonUser', TRUE);
    }

    $addCaptcha = array();
    $emailPresent = FALSE;

    // cache the state country fields. based on the results, we could use our javascript solution
    // in create or register mode
    $stateCountryMap = array();

    // add the form elements
    foreach ($this->_fields as $name => $field) {
      // make sure that there is enough permission to expose this field
      if (!$admin && $field['visibility'] == 'User and User Admin Only') {
        unset($this->_fields[$name]);
        continue;
      }

      // since the CMS manages the email field, suppress the email display if in
      // register mode which occur within the CMS form
      if ($this->_mode == self::MODE_REGISTER &&
        substr($name, 0, 5) == 'email'
      ) {
        unset($this->_fields[$name]);
        continue;
      }

      list($prefixName, $index) = CRM_Utils_System::explode('-', $name, 2);
      if ($prefixName == 'state_province' || $prefixName == 'country' || $prefixName == 'county') {
        if (!array_key_exists($index, $stateCountryMap)) {
          $stateCountryMap[$index] = array();
        }
        $stateCountryMap[$index][$prefixName] = $name;
      }

      CRM_Core_BAO_UFGroup::buildProfile($this, $field, $this->_mode);

      if ($field['add_to_group_id']) {
        $addToGroupId = $field['add_to_group_id'];
      }

      //build array for captcha
      if ($field['add_captcha']) {
        $addCaptcha[$field['group_id']] = $field['add_captcha'];
      }

      if (($name == 'email-Primary') || ($name == 'email-' . isset($primaryLocationType) ? $primaryLocationType : "")) {
        $emailPresent = TRUE;
        $this->_mail = $name;
      }
    }

    // add captcha only for create mode.
    if ($this->_mode == self::MODE_CREATE) {
      if (!$this->_isAddCaptcha && !empty($addCaptcha)) {
        $this->_isAddCaptcha = TRUE;
      }
      if ($this->_gid) {
        $dao = new CRM_Core_DAO_UFGroup();
        $dao->id = $this->_gid;
        $dao->addSelect();
        $dao->addSelect('add_captcha', 'is_update_dupe');
        if ($dao->find(TRUE)) {
          if ($dao->add_captcha) {
            $setCaptcha = TRUE;
          }
          if ($dao->is_update_dupe) {
            $this->_isUpdateDupe = $dao->is_update_dupe;
          }
        }
      }
    }
    else {
      $this->_isAddCaptcha = FALSE;
    }

    //finally add captcha to form.
    if ($this->_isAddCaptcha) {
      $captcha = CRM_Utils_ReCAPTCHA::singleton();
      $captcha->add($this);
    }
    $this->assign("isCaptcha", $this->_isAddCaptcha);

    if ($this->_mode != self::MODE_SEARCH) {
      if (isset($addToGroupId)) {
        $this->add('hidden', "add_to_group", $addToGroupId);
        $this->_addToGroupID = $addToGroupId;
      }
    }

    // also do state country js
    CRM_Core_BAO_Address::addStateCountryMap($stateCountryMap, $this->_defaults);

    $action = CRM_Utils_Request::retrieve('action', 'String', $this, FALSE, NULL);
    if ($this->_mode == self::MODE_CREATE) {
      CRM_Core_BAO_CMSUser::buildForm($this, $this->_gid, $emailPresent, $action);
    }
    else {
      $this->assign('showCMS', FALSE);
    }

    $this->assign('groupId', $this->_gid);

    // now fix all state country selectors
    CRM_Core_BAO_Address::fixAllStateSelects($this, $this->_defaults);

    // if view mode pls freeze it with the done button.
    if ($this->_action & CRM_Core_Action::VIEW) {
      $this->freeze();
    }

    if ($this->_context == 'dialog') {
      $this->addElement('submit',
        $this->_duplicateButtonName,
        ts('Save Matching Contact')
      );
    }

    $this->setDefaultsValues();
  }

  /*
     * Function to validate profile and provided activity Id
     *                                          
     * @params Integer $activityId Activity Id
     * @params Integer $gid        Profile Id
     *
     * @return Array   $errors     Errors ( if any ).
     */

  static
  function validateContactActivityProfile($activityId, $contactId, $gid) {
    $errors = array();
    if (!$activityId) {
      $errors[] = 'Profile is using one or more activity fields, and is missing the activity Id (aid) in the URL.';
      return $errors;
    }

    $activityDetails = array();
    $activityParams = array('id' => $activityId);
    CRM_Activity_BAO_Activity::retrieve($activityParams, $activityDetails);

    if (empty($activityDetails)) {
      $errors[] = 'Invalid Activity Id (aid).';
      return $errors;
    }

    $profileActivityTypes = CRM_Core_BAO_UFGroup::groupTypeValues($gid, 'Activity');

    if ((CRM_Utils_Array::value('Activity', $profileActivityTypes) &&
        !in_array($activityDetails['activity_type_id'], $profileActivityTypes['Activity'])
      ) ||
      (!in_array($contactId, $activityDetails['assignee_contact']) &&
        !in_array($contactId, $activityDetails['target_contact'])
      )
    ) {
      $errors[] = 'This activity cannot be edited or viewed via this profile.';
    }

    return $errors;
  }

  /**
   * global form rule
   *
   * @param array  $fields the input form values
   * @param array  $files  the uploaded files if any
   * @param object $form   the form object
   *
   * @return true if no errors, else array of errors
   * @access public
   * @static
   */
  static
  function formRule($fields, $files, $form) {
    $errors = array();
    // if no values, return
    if (empty($fields)) {
      return TRUE;
    }

    $cid = $register = NULL;

    // hack we use a -1 in options to indicate that its registration
    if ($form->_id) {
      $cid = $form->_id;
      $form->_isUpdateDupe = 1;
    }

    if ($form->_mode == CRM_Profile_Form::MODE_REGISTER) {
      $register = TRUE;
    }

    // dont check for duplicates during registration validation: CRM-375
    if (!$register && !CRM_Utils_Array::value('_qf_Edit_upload_duplicate', $fields)) {
      // fix for CRM-3240
      if (CRM_Utils_Array::value('email-Primary', $fields)) {
        $fields['email'] = CRM_Utils_Array::value('email-Primary', $fields);
      }

      // fix for CRM-6141
      if (CRM_Utils_Array::value('phone-Primary-1', $fields) &&
        !CRM_Utils_Array::value('phone-Primary', $fields)
      ) {
        $fields['phone-Primary'] = $fields['phone-Primary-1'];
      }

      $session = CRM_Core_Session::singleton();

      $ctype = CRM_Core_BAO_UFGroup::getContactType($form->_gid);
      // If all profile fields is of Contact Type then consider
      // profile is of Individual type(default).
      if (!$ctype) {
        $ctype = 'Individual';
      }
      $dedupeParams = CRM_Dedupe_Finder::formatParams($fields, $ctype);
      if ($form->_mode == CRM_Profile_Form::MODE_CREATE) {
        // fix for CRM-2888
        $exceptions = array();
      }
      else {
        // for edit mode we need to allow our own record to be a dupe match!
        $exceptions = array($session->get('userID'));
      }

      // for dialog mode we should always use fuzzy rule.
      $ruleType = 'Strict';
      if ($form->_context == 'dialog') {
        $ruleType = 'Fuzzy';
      }

      $dedupeParams['check_permission'] = FALSE;
      $ids = CRM_Dedupe_Finder::dupesByParams($dedupeParams,
        $ctype,
        $ruleType,
        $exceptions
      );
      if ($ids) {
        if ($form->_isUpdateDupe == 2) {
          CRM_Core_Session::setStatus(ts('Note: this contact may be a duplicate of an existing record.'));
        }
        elseif ($form->_isUpdateDupe == 1) {
          if (!$form->_id) {
            $form->_id = $ids[0];
          }
        }
        else {
          if ($form->_context == 'dialog') {

            $contactLinks = CRM_Contact_BAO_Contact_Utils::formatContactIDSToLinks($ids, TRUE, TRUE);

            $duplicateContactsLinks = '<div class="matching-contacts-found">';
            $duplicateContactsLinks .= ts('One matching contact was found. ', array('count' => count($contactLinks['rows']), 'plural' => '%count matching contacts were found.<br />'));
            if ($contactLinks['msg'] == 'view') {
              $duplicateContactsLinks .= ts('You can View the existing contact.', array('count' => count($contactLinks['rows']), 'plural' => 'You can View the existing contacts.'));
            }
            else {
              $duplicateContactsLinks .= ts('You can View or Edit the existing contact.', array('count' => count($contactLinks['rows']), 'plural' => 'You can View or Edit the existing contacts.'));
            }
            $duplicateContactsLinks .= '</div>';
            $duplicateContactsLinks .= '<table class="matching-contacts-actions">';
            $row = '';
            for ($i = 0; $i < sizeof($contactLinks['rows']); $i++) {
              $row .= '  <tr>	 ';
              $row .= '  	<td class="matching-contacts-name"> ';
              $row .= $contactLinks['rows'][$i]['display_name'];
              $row .= '  	</td>';
              $row .= '  	<td class="matching-contacts-email"> ';
              $row .= $contactLinks['rows'][$i]['primary_email'];
              $row .= '  	</td>';
              $row .= '  	<td class="action-items"> ';
              $row .= $contactLinks['rows'][$i]['view'] . ' ';
              $row .= $contactLinks['rows'][$i]['edit'];
              $row .= '  	</td>';
              $row .= '  </tr>	 ';
            }

            $duplicateContactsLinks .= $row . '</table>';
            $duplicateContactsLinks .= "If you're sure this record is not a duplicate, click the 'Save Matching Contact' button below.";

            $errors['_qf_default'] = $duplicateContactsLinks;


            // let smarty know that there are duplicates
            $template = CRM_Core_Smarty::singleton();
            $template->assign('isDuplicate', 1);
          }
          else {
            $errors['_qf_default'] = ts('A record already exists with the same information.');
          }
        }
      }
    }

    foreach ($fields as $key => $value) {
      list($fieldName, $locTypeId, $phoneTypeId) = CRM_Utils_System::explode('-', $key, 3);
      if ($fieldName == 'state_province' && $fields["country-{$locTypeId}"]) {
        // Validate Country - State list
        $countryId = $fields["country-{$locTypeId}"];
        $stateProvinceId = $value;

        if ($stateProvinceId && $countryId) {
          $stateProvinceDAO = new CRM_Core_DAO_StateProvince();
          $stateProvinceDAO->id = $stateProvinceId;
          $stateProvinceDAO->find(TRUE);

          if ($stateProvinceDAO->country_id != $countryId) {
            // country mismatch hence display error
            $stateProvinces = CRM_Core_PseudoConstant::stateProvince();
            $countries      = CRM_Core_PseudoConstant::country();
            $errors[$key]   = "State/Province " . $stateProvinces[$stateProvinceId] . " is not part of " . $countries[$countryId] . ". It belongs to " . $countries[$stateProvinceDAO->country_id] . ".";
          }
        }
      }

      if ($fieldName == 'county' && $fields["state_province-{$locTypeId}"]) {
        // Validate County - State list
        $stateProvinceId = $fields["state_province-{$locTypeId}"];
        $countyId = $value;

        if ($countyId && $stateProvinceId) {
          $countyDAO = new CRM_Core_DAO_County();
          $countyDAO->id = $countyId;
          $countyDAO->find(TRUE);

          if ($countyDAO->state_province_id != $stateProvinceId) {
            // state province mismatch hence display error
            $stateProvinces = CRM_Core_PseudoConstant::stateProvince();
            $counties       = CRM_Core_PseudoConstant::county();
            $errors[$key]   = "County " . $counties[$countyId] . " is not part of " . $stateProvinces[$stateProvinceId] . ". It belongs to " . $stateProvinces[$countyDAO->state_province_id] . ".";
          }
        }
      }
    }
    foreach (CRM_Contact_BAO_Contact::$_greetingTypes as $greeting) {
      if ($greetingType = CRM_Utils_Array::value($greeting, $fields)) {
        $customizedValue = CRM_Core_OptionGroup::getValue($greeting, 'Customized', 'name');
        if ($customizedValue == $greetingType && empty($fields[$greeting . '_custom'])) {
          $errors[$greeting . '_custom'] = ts('Custom  %1 is a required field if %1 is of type Customized.',
            array(1 => ucwords(str_replace('_', ' ', $greeting)))
          );
        }
      }
    }

    return empty($errors) ? TRUE : $errors;
  }

  /**
   * Process the user submitted custom data values.
   *
   * @access public
   *
   * @return void
   */
  public function postProcess() {
    $params = $this->controller->exportValues($this->_name);

    if (CRM_Utils_Array::value('image_URL', $params)) {
      CRM_Contact_BAO_Contact::processImageParams($params);
    }

    $greetingTypes = array(
      'addressee' => 'addressee_id',
      'email_greeting' => 'email_greeting_id',
      'postal_greeting' => 'postal_greeting_id',
    );
    if ($this->_id) {
      $contactDetails = CRM_Contact_BAO_Contact::getHierContactDetails($this->_id,
        $greetingTypes
      );
      $details = $contactDetails[0][$this->_id];
    }
    if (!(CRM_Utils_Array::value('addressee_id', $details) ||
        CRM_Utils_Array::value('email_greeting_id', $details) ||
        CRM_Utils_Array::value('postal_greeting_id', $details)
      )) {

      $profileType = CRM_Core_BAO_UFField::getProfileType($this->_gid);
      //Though Profile type is contact we need
      //Individual/Household/Organization for setting Greetings.
      if ($profileType == 'Contact') {
        $profileType = 'Individual';
        //if we editing Household/Organization.
        if ($this->_id) {
          $profileType = CRM_Contact_BAO_Contact::getContactType($this->_id);
        }
      }
      if (CRM_Contact_BAO_ContactType::isaSubType($profileType)) {
        $profileType = CRM_Contact_BAO_ContactType::getBasicType($profileType);
      }

      foreach ($greetingTypes as $key => $value) {
        if (!array_key_exists($key, $params)) {
          $params[$key] = CRM_Contact_BAO_Contact_Utils::defaultGreeting($profileType, $key);
        }
      }
    }
    if ($this->_mode == self::MODE_REGISTER) {
      CRM_Core_BAO_Address::setOverwrite(FALSE);
    }

    $transaction = new CRM_Core_Transaction();

    //used to send subcribe mail to the group which user want.
    //if the profile double option in is enabled
    $mailingType = array();
    $config = CRM_Core_Config::singleton();

    $result = NULL;
    foreach ($params as $name => $values) {
      if (substr($name, 0, 6) == 'email-') {
        $result['email'] = $values;
      }
    }

    //array of group id, subscribed by contact
    $contactGroup = array();
    if (CRM_Utils_Array::value('group', $params) &&
      CRM_Core_BAO_UFGroup::isProfileDoubleOptin()
    ) {
      $groupSubscribed = array();
      if (CRM_Utils_Array::value('email', $result)) {
        if ($this->_id) {
          $contactGroups = new CRM_Contact_DAO_GroupContact();
          $contactGroups->contact_id = $this->_id;
          $contactGroups->status = 'Added';
          $contactGroups->find();
          $contactGroup = array();
          while ($contactGroups->fetch()) {
            $contactGroup[] = $contactGroups->group_id;
            $groupSubscribed[$contactGroups->group_id] = 1;
          }
        }
        foreach ($params['group'] as $key => $val) {
          if (!$val) {
            unset($params['group'][$key]);
            continue;
          }
          $groupTypes = CRM_Core_DAO::getFieldValue('CRM_Contact_DAO_Group',
            $key, 'group_type', 'id'
          );
          $groupType = explode(CRM_Core_DAO::VALUE_SEPARATOR,
            substr($groupTypes, 1, -1)
          );
          //filter group of mailing type and unset it from params
          if (in_array(2, $groupType)) {
            //if group is already subscribed , ignore it
            $groupExist = CRM_Utils_Array::key($key, $contactGroup);
            if (!isset($groupExist)) {
              $mailingType[] = $key;
              unset($params['group'][$key]);
            }
          }
        }
      }
    }

    $addToGroupId = NULL;
    if (CRM_Utils_Array::value('add_to_group', $params)) {
      $addToGroupId = $params['add_to_group'];

      //run same check whether group is a mailing list
      $groupTypes = CRM_Core_DAO::getFieldValue('CRM_Contact_DAO_Group',
        $addToGroupId, 'group_type', 'id'
      );
      $groupType = explode(CRM_Core_DAO::VALUE_SEPARATOR,
        substr($groupTypes, 1, -1)
      );
      //filter group of mailing type and unset it from params
      if (in_array(2, $groupType) &&
        CRM_Utils_Array::value('email', $result) &&
        CRM_Core_BAO_UFGroup::isProfileAddToGroupDoubleOptin()
      ) {
        if (!count($contactGroup)) {
          //array of group id, subscribed by contact
          $contactGroup = array();
          if ($this->_id) {
            $contactGroups = new CRM_Contact_DAO_GroupContact();
            $contactGroups->contact_id = $this->_id;
            $contactGroups->status = 'Added';
            $contactGroups->find();
            $contactGroup = array();
            while ($contactGroups->fetch()) {
              $contactGroup[] = $contactGroups->group_id;
              $groupSubscribed[$contactGroups->group_id] = 1;
            }
          }
        }
        //if group is already subscribed , ignore it
        $groupExist = CRM_Utils_Array::key($addToGroupId, $contactGroup);
        if (!isset($groupExist)) {
          $mailingType[] = $addToGroupId;
          $addToGroupId = NULL;
        }
      }
      else {
        // since we are directly adding contact to group lets unset it from mailing
        if ($key = array_search($addToGroupId, $mailingType)) {
          unset($mailingType[$key]);
        }
      }
    }

    if ($this->_grid) {
      $params['group'] = $groupSubscribed;
    }

    // commenting below code, since we potentially
    // triggered maximum name field formatting cases during CRM-4430.
    // CRM-4343
    // $params['preserveDBName'] = true;

    $profileFields = $this->_fields;
    if (($this->_mode & self::MODE_EDIT) && $this->_activityId && $this->_isContactActivityProfile) {
      $profileFields = $activityParams = array();
      foreach ($this->_fields as $fieldName => $field) {
        if (CRM_Utils_Array::value('field_type', $field) == 'Activity') {
          if (isset($params[$fieldName])) {
            $activityParams[$fieldName] = $params[$fieldName];
          }
          if (isset($params['activity_date_time'])) {
            $activityParams['activity_date_time'] = CRM_Utils_Date::processDate($params['activity_date_time'], $params['activity_date_time_time']);
          }
          if (CRM_Utils_Array::value($fieldName, $params) && isset($params["{$fieldName}_id"])) {
            $activityParams[$fieldName] = $params["{$fieldName}_id"];
          }
        }
        else {
          $profileFields[$fieldName] = $field;
        }
      }

      if (!empty($activityParams)) {
        $activityParams['version'] = 3;
        $activityParams['id'] = $this->_activityId;
        $activityParams['skipRecentView'] = TRUE;
        $activity = civicrm_api('Activity', 'create', $activityParams);
      }
    }

    $this->_id = CRM_Contact_BAO_Contact::createProfileContact($params, $profileFields,
      $this->_id, $addToGroupId,
      $this->_gid, $this->_ctype,
      TRUE
    );
    //mailing type group
    if (!empty($mailingType)) {
      // we send in the contactID so we match the same groups and are exact, rather than relying on email
      // CRM-8710
      CRM_Mailing_Event_BAO_Subscribe::commonSubscribe($mailingType, $result, $this->_id, 'profile');
    }

    $ufGroups = array();
    if ($this->_gid) {
      $ufGroups[$this->_gid] = 1;
    }
    elseif ($this->_mode == self::MODE_REGISTER) {
      $ufGroups = CRM_Core_BAO_UFGroup::getModuleUFGroup('User Registration');
    }

    foreach ($ufGroups as $gId => $val) {
      if ($notify = CRM_Core_DAO::getFieldValue('CRM_Core_DAO_UFGroup', $gId, 'notify')) {
        $values = CRM_Core_BAO_UFGroup::checkFieldsEmptyValues($gId, $this->_id, NULL);
        CRM_Core_BAO_UFGroup::commonSendMail($this->_id, $values);
      }
    }

    //create CMS user (if CMS user option is selected in profile)
    if (CRM_Utils_Array::value('cms_create_account', $params) &&
      $this->_mode == self::MODE_CREATE
    ) {
      $params['contactID'] = $this->_id;
      if (!CRM_Core_BAO_CMSUser::create($params, $this->_mail)) {
        CRM_Core_Session::setStatus(ts('Your profile is not saved and Account is not created.'));
        $transaction->rollback();
        return CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/profile/create',
            'reset=1&gid=' . $this->_gid
          ));
      }
    }

    $transaction->commit();
  }

  function checkTemplateFileExists($suffix = NULL) {
    if ($this->_gid) {
      $templateFile = "CRM/Profile/Form/{$this->_gid}/{$this->_name}.{$suffix}tpl";
      $template = CRM_Core_Form::getTemplate();
      if ($template->template_exists($templateFile)) {
        return $templateFile;
      }

      // lets see if we have customized by name
      $ufGroupName = CRM_Core_DAO::getFieldValue('CRM_Core_DAO_UFGroup', $this->_gid, 'name');
      if ($ufGroupName) {
        $templateFile = "CRM/Profile/Form/{$ufGroupName}/{$this->_name}.{$suffix}tpl";
        if ($template->template_exists($templateFile)) {
          return $templateFile;
        }
      }
    }
    return NULL;
  }

  function getTemplateFileName() {
    $fileName = $this->checkTemplateFileExists();
    return $fileName ? $fileName : parent::getTemplateFileName();
  }

  function overrideExtraTemplateFileName() {
    $fileName = $this->checkTemplateFileExists('extra.');
    return $fileName ? $fileName : parent::overrideExtraTemplateFileName();
  }
}

